@@ -7,14 +7,13 @@
 #define SHADOW_INCLUDE_LIB_ATOI_GETNUM_H_
 
 
-#include <config.h>
+#include "config.h"
 
 #include <limits.h>
 #include <stddef.h>
 #include <sys/types.h>
 
-#include "atoi/a2i/a2i.h"
-#include "atoi/a2i/a2s.h"
+#include "atoi/a2i.h"
 #include "attr.h"
 #include "typetraits.h"
 
