@@ -5,7 +5,7 @@
 #define _SHADOW_INCLUDE_LIB_FIELDS_H_
 
 
-#include <config.h>
+#include "config.h"
 
 #include <stddef.h>
 
