@@ -6,7 +6,7 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <assert.h>
 #include <stdint.h>
@@ -18,16 +18,18 @@
 #include "groupio.h"
 #include "getdef.h"
 #include "shadowlog.h"
+#include "string/strerrno.h"
+
 
 /*
  * get_ranges - Get the minimum and maximum ID ranges for the search
  *
  * This function will return the minimum and maximum ranges for IDs
  *
- * 0: The function completed successfully
- * EINVAL: The provided ranges are impossible (such as maximum < minimum)
+ * 0: the function completed successfully
+ * EINVAL: the provided ranges are impossible (such as maximum < minimum)
  *
- * preferred_min: The special-case minimum value for a specifically-
+ * preferred_min: the special-case minimum value for a specifically-
  * requested ID, which may be lower than the standard min_id
  */
 static int get_ranges (bool sys_group, gid_t *min_id, gid_t *max_id,
@@ -54,11 +56,11 @@ static int get_ranges (bool sys_group, gid_t *min_id, gid_t *max_id,
 		/* Check that the ranges make sense */
 		if (*max_id < *min_id) {
 			(void) fprintf (log_get_logfd(),
-                            _("%s: Invalid configuration: SYS_GID_MIN (%lu), "
-                              "GID_MIN (%lu), SYS_GID_MAX (%lu)\n"),
-                            log_get_progname(), (unsigned long) *min_id,
-                            getdef_ulong ("GID_MIN", 1000UL),
-                            (unsigned long) *max_id);
+			    _("%s: Invalid configuration: SYS_GID_MIN (%lu), "
+			      "GID_MIN (%lu), SYS_GID_MAX (%lu)\n"),
+			    log_get_progname(), (unsigned long) *min_id,
+			    getdef_ulong ("GID_MIN", 1000UL),
+			    (unsigned long) *max_id);
 			return EINVAL;
 		}
 		/*
@@ -233,11 +235,11 @@ int find_new_gid (bool sys_group,
 	 */
 
 	/* Create an array to hold all of the discovered GIDs */
-	used_gids = CALLOC (gid_max + 1, bool);
+	used_gids = calloc_T(gid_max + 1, bool);
 	if (NULL == used_gids) {
 		fprintf (log_get_logfd(),
 			 _("%s: failed to allocate memory: %s\n"),
-			 log_get_progname(), strerror (errno));
+			 log_get_progname(), strerrno());
 		return -1;
 	}
 
@@ -245,7 +247,7 @@ int find_new_gid (bool sys_group,
 	(void) gr_rewind ();
 	highest_found = gid_min;
 	lowest_found = gid_max;
-	while ((grp = gr_next ()) != NULL) {
+	while (NULL != (grp = gr_next())) {
 		/*
 		 * Does this entry have a lower GID than the lowest we've found
 		 * so far?
