@@ -9,6 +9,12 @@
 #ifndef _GETDEF_H
 #define _GETDEF_H
 
+
+#include "config.h"
+
+#include <stdbool.h>
+
+
 /* getdef.c */
 extern bool getdef_bool (const char *);
 extern long getdef_long (const char *, long);
