@@ -1,4 +1,4 @@
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -109,14 +109,14 @@ nss_init(const char *nsswitch_path) {
 		fprintf(shadow_logfd, "Using files\n");
 		goto null_subid;
 	}
-	SNPRINTF(libname, "libsubid_%s.so", p);
+	stprintf_a(libname, "libsubid_%s.so", p);
 	h = dlopen(libname, RTLD_LAZY);
 	if (!h) {
 		fprintf(shadow_logfd, "Error opening %s: %s\n", libname, dlerror());
 		fprintf(shadow_logfd, "Using files\n");
 		goto null_subid;
 	}
-	subid_nss = MALLOC(1, struct subid_nss_ops);
+	subid_nss = malloc_T(1, struct subid_nss_ops);
 	if (!subid_nss) {
 		goto close_lib;
 	}
