@@ -14,8 +14,9 @@
 
 #include <sys/types.h>
 #include <pwd.h>
+#include <stdbool.h>
 
-extern int pw_close (void);
+extern int pw_close (bool process_selinux);
 extern /*@observer@*/ /*@null@*/const struct passwd *pw_locate (const char *name);
 extern /*@observer@*/ /*@null@*/const struct passwd *pw_locate_uid (uid_t uid);
 extern int pw_lock (void);
@@ -25,7 +26,7 @@ extern /*@observer@*/ /*@null@*/const struct passwd *pw_next (void);
 extern int pw_open (int mode);
 extern int pw_remove (const char *name);
 extern int pw_rewind (void);
-extern int pw_unlock (void);
+extern int pw_unlock (bool process_selinux);
 extern int pw_update (const struct passwd *pw);
 extern int pw_sort (void);
 
