@@ -2,6 +2,6 @@
 // SPDX-License-Identifier: BSD-3-Clause
 
 
-#include <config.h>
+#include "config.h"
 
 #include "string/strdup/strndupa.h"
