@@ -5,7 +5,7 @@
 #ifndef _SUBORDINATEIO_H
 #define _SUBORDINATEIO_H
 
-#include <config.h>
+#include "config.h"
 
 #ifdef ENABLE_SUBIDS
 
@@ -13,7 +13,7 @@
 
 #include "../libsubid/subid.h"
 
-extern int sub_uid_close(void);
+extern int sub_uid_close(bool process_selinux);
 extern bool have_sub_uids(const char *owner, uid_t start, unsigned long count);
 extern bool sub_uid_file_present (void);
 extern bool local_sub_uid_assigned(const char *owner);
@@ -21,17 +21,18 @@ extern int sub_uid_lock (void);
 extern int sub_uid_setdbname (const char *filename);
 extern /*@observer@*/const char *sub_uid_dbname (void);
 extern int sub_uid_open (int mode);
-extern int sub_uid_unlock (void);
+extern int sub_uid_unlock (bool process_selinux);
 extern int sub_uid_add (const char *owner, uid_t start, unsigned long count);
 extern int sub_uid_remove (const char *owner, uid_t start, unsigned long count);
 extern uid_t sub_uid_find_free_range(uid_t min, uid_t max, unsigned long count);
+extern bool want_subuid_file(void);
 extern int list_owner_ranges(const char *owner, enum subid_type id_type, struct subid_range **ranges);
 extern bool new_subid_range(struct subordinate_range *range, enum subid_type id_type, bool reuse);
 extern bool release_subid_range(struct subordinate_range *range, enum subid_type id_type);
 extern int find_subid_owners(unsigned long id, enum subid_type id_type, uid_t **uids);
 extern void free_subordinate_ranges(struct subordinate_range **ranges, int count);
 
-extern int sub_gid_close(void);
+extern int sub_gid_close(bool process_selinux);
 extern bool have_sub_gids(const char *owner, gid_t start, unsigned long count);
 extern bool sub_gid_file_present (void);
 extern bool local_sub_gid_assigned(const char *owner);
@@ -39,10 +40,11 @@ extern int sub_gid_lock (void);
 extern int sub_gid_setdbname (const char *filename);
 extern /*@observer@*/const char *sub_gid_dbname (void);
 extern int sub_gid_open (int mode);
-extern int sub_gid_unlock (void);
+extern int sub_gid_unlock (bool process_selinux);
 extern int sub_gid_add (const char *owner, gid_t start, unsigned long count);
 extern int sub_gid_remove (const char *owner, gid_t start, unsigned long count);
 extern uid_t sub_gid_find_free_range(gid_t min, gid_t max, unsigned long count);
+extern bool want_subgid_file(void);
 
 extern void free_subid_pointer(void *ptr);
 
