@@ -6,7 +6,7 @@
 #define SHADOW_INCLUDE_LIB_TYPETRAITS_H_
 
 
-#include <config.h>
+#include "config.h"
 
 #include "sizeof.h"
 
@@ -54,4 +54,15 @@
 )
 
 
+#define QChar_of(s)  typeof                                           \
+(                                                                     \
+	_Generic(s,                                                   \
+		const char *:  (const char){0},                       \
+		const void *:  (const char){0},                       \
+		char *:        (char){0},                             \
+		void *:        (char){0}                              \
+	)                                                             \
+)
+
+
 #endif  // include guard
