@@ -29,10 +29,11 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <errno.h>
 
 #include "alloc/malloc.h"
-#include "alloc/x/xrealloc.h"
+#include "alloc/realloc.h"
 #include "prototypes.h"
 #include "shadowlog.h"
 
@@ -53,7 +54,7 @@
 	/* we have to start with something */
 	size_t length = 0x100;
 
-	result = MALLOC(1, LOOKUP_TYPE);
+	result = malloc_T(1, LOOKUP_TYPE);
 	if (NULL == result) {
 		goto oom;
 	}
@@ -61,7 +62,7 @@
 	while (true) {
 		int status;
 		LOOKUP_TYPE *resbuf = NULL;
-		buffer = XREALLOC(buffer, length, char);
+		buffer = xrealloc_T(buffer, length, char);
 		status = REENTRANT_NAME(ARG_NAME, result, buffer,
 		                        length, &resbuf);
 		if ((0 == status) && (resbuf == result)) {
