@@ -3,7 +3,7 @@
 #include <stdio.h>
 #include <unistd.h>
 
-#include "atoi/str2i.h"
+#include "atoi/a2i.h"
 #include "subid.h"
 #include "stdlib.h"
 #include "prototypes.h"
