@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -20,6 +20,7 @@
 #include "prototypes.h"
 #include "shadowlog.h"
 #include "string/strcmp/strprefix.h"
+#include "string/strerrno.h"
 #include "subordinateio.h"
 
 
@@ -103,8 +104,7 @@ static void write_setgroups(int proc_dir_fd, bool allow_setgroups)
 			goto out;
 		}
 		fprintf(stderr, _("%s: couldn't open process setgroups: %s\n"),
-			Prog,
-			strerror(errno));
+			Prog, strerrno());
 		exit(EXIT_FAILURE);
 	}
 
@@ -115,8 +115,7 @@ static void write_setgroups(int proc_dir_fd, bool allow_setgroups)
 	 */
 	if (read(setgroups_fd, policy_buffer, sizeof(policy_buffer)) < 0) {
 		fprintf(stderr, _("%s: failed to read setgroups: %s\n"),
-			Prog,
-			strerror(errno));
+			Prog, strerrno());
 		exit(EXIT_FAILURE);
 	}
 	if (strprefix(policy_buffer, policy))
@@ -125,15 +124,12 @@ static void write_setgroups(int proc_dir_fd, bool allow_setgroups)
 	/* Write the policy. */
 	if (lseek(setgroups_fd, 0, SEEK_SET) < 0) {
 		fprintf(stderr, _("%s: failed to seek setgroups: %s\n"),
-			Prog,
-			strerror(errno));
+			Prog, strerrno());
 		exit(EXIT_FAILURE);
 	}
 	if (dprintf(setgroups_fd, "%s", policy) < 0) {
 		fprintf(stderr, _("%s: failed to setgroups %s policy: %s\n"),
-			Prog,
-			policy,
-			strerror(errno));
+			Prog, policy, strerrno());
 		exit(EXIT_FAILURE);
 	}
 
@@ -195,7 +191,7 @@ int main(int argc, char **argv)
 	if (fstat(proc_dir_fd, &st) < 0) {
 		fprintf(stderr,
 		        _("%s: Could not stat directory for target process: %s\n"),
-		        Prog, strerror (errno));
+		        Prog, strerrno());
 		return EXIT_FAILURE;
 	}
 
@@ -214,10 +210,10 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	if (!sub_gid_open(O_RDONLY)) {
+	if (want_subgid_file() && !sub_gid_open(O_RDONLY)) {
 		fprintf (stderr,
 		         _("%s: cannot open %s: %s\n"),
-		         Prog, sub_gid_dbname (), strerror (errno));
+		         Prog, sub_gid_dbname(), strerrno());
 		return EXIT_FAILURE;
 	}
 
@@ -230,7 +226,8 @@ int main(int argc, char **argv)
 
 	write_setgroups(proc_dir_fd, allow_setgroups);
 	write_mapping(proc_dir_fd, ranges, mappings, "gid_map", pw->pw_uid);
-	sub_gid_close();
+	if (want_subgid_file())
+		sub_gid_close(true);
 
 	return EXIT_SUCCESS;
 }
